import Qt 4.7

Item {
    id: rectangle
    anchors.fill: parent

    property alias text: textField.text
    property alias font: textField.font
    property alias horizontalAlignment: textField.horizontalAlignment
    property color textColor: "white"
    //property color activatedColor: "#222222"
    property color activatedColor: "white"
    property color deactivatedColor: "transparent"
    property int tabOrder: -1
    property bool editable: true
    property bool heightAdjustedByFontSize: false
    property int backgroundRadius: 0

    property bool isActive: false

    property alias mouseAreaVisible: textFieldMouseArea.visible

    signal textChanged()
    signal selectReceived()
    signal focusReceived()
    signal focusLost()
    signal returnPressed()
    signal tabPressed()
    signal backtabPressed()
    signal escapePressed()

    function deactivateEdit() {
        deactivateControl()
    }

    function activateControl(cursorVisible) {
        //console.debug("QML++ activateControl: " + rectangle.text)

        cursorVisible = (typeof cursorVisible == 'undefined') ? false : cursorVisible

        textField.enabled = true
        textFieldMouseArea.visible = false

        textField.cursorVisible = cursorVisible
        textField.color = "black"
        //content.color = rectangle.activatedColor
        //textFieldBorderRect.color = "white"
        content.color = "transparent"
        textFieldBorderRect.color = rectangle.activatedColor
        textFieldBorderRect.border.color = "#4d9adf"

        rectangle.isActive = true

        textField.focus = true
        textField.selectAll()
        textField.visible = true
    }

    function deactivateControl() {
        //console.debug("QML++ deactivateControl: " + rectangle.text)

        textField.cursorVisible = false
        textField.color = rectangle.textColor
        content.color = rectangle.deactivatedColor
        textFieldBorderRect.color = rectangle.deactivatedColor
        textFieldBorderRect.border.color = "transparent"

        content.focus = false
        textField.focus = false
        textField.select(0,0)

        textField.enabled = false
        textField.visible = false
        textFieldMouseArea.visible = true

        rectangle.isActive = false
    }

    function setFocus() {
        //console.debug("QML++ setFocus")

        rectangle.focusReceived()
    }

    Rectangle {
        id: content
        anchors.left: parent.left
        anchors.top: parent.top
        anchors.right: parent.right
        height: (heightAdjustedByFontSize) ? textField.height + 10 : parent.height
        //color: rectangle.deactivatedColor
        color: "transparent"
        radius: rectangle.backgroundRadius
        smooth: false
        opacity: 1.0

        Rectangle {
            id: textFieldBorderRect
            anchors.left: parent.left
            anchors.leftMargin: 5
            anchors.top: parent.top
            anchors.topMargin: (heightAdjustedByFontSize) ? 0 : 4
            anchors.right: parent.right
            anchors.rightMargin: 5
            height: (heightAdjustedByFontSize) ? textField.height + 8 : parent.height - 6
            color: rectangle.deactivatedColor

            border.width: 1
            border.color: rectangle.deactivatedColor

            smooth: false

            TextInput {
                id: textField
                text: ""

                color: rectangle.textColor
                anchors.top: parent.top
                //anchors.topMargin: (heightAdjustedByFontSize) ? 2 : (parent.height - font.pixelSize) / 2
                anchors.topMargin: Math.ceil((parent.height - height) / 2)
                anchors.left: parent.left
                anchors.leftMargin: 2
                anchors.right: parent.right
                anchors.rightMargin: 2
                //height: (heightAdjustedByFontSize) ? font.pixelSize : parent.height - 4

                //font.pixelSize: 11
                selectionColor: "#b9d4fe"

                smooth: true
                horizontalAlignment: Text.AlignLeft

                cursorVisible: false
                selectByMouse: true

                focus: rectangle.focus
                enabled: rectangle.editable
                activeFocusOnPress: rectangle.editable
                visible: false

                onFocusChanged: {
                    //console.debug("QML: ItemViewTextInput textField onFocusChanged with " + rectangle.text)
                    if (focus) {
                        rectangle.focusReceived()

                    } else {
                        textField.select(0,0)
                        rectangle.focusLost();
                    }
                }

                onTextChanged: rectangle.textChanged()

                Keys.onReturnPressed: rectangle.returnPressed()
                Keys.onTabPressed: rectangle.tabPressed()
                Keys.onBacktabPressed: rectangle.backtabPressed()
                Keys.onEscapePressed: rectangle.escapePressed()

            }

            Text {
                id: textPreview
                text: textField.text

                color: rectangle.textColor
                anchors.top: textField.top
                anchors.left: textField.left
                anchors.right: textField.right

                font: textField.font
                smooth: true
                horizontalAlignment: Text.AlignLeft
                elide: Text.ElideRight

                visible: !textField.visible
            }

            MouseArea {
                id: textFieldMouseArea
                acceptedButtons: Qt.LeftButton
                visible: !textField.cursorVisible
                enabled: visible

                anchors.fill: textField
                onDoubleClicked: {
                    //console.debug("QML: ItemViewTextInput textFieldMouseArea onDoubleClicked with " + rectangle.text)
                    //console.debug(rectangle.editable)
//                    if (rectangle.editable) {
                        rectangle.setFocus()
                        rectangle.selectReceived()
//                    }
                }

                onPressed: {
                    //console.debug("QML: ItemViewTextInput textFieldMouseArea onPressed with " + rectangle.text)
                    rectangle.selectReceived()
                }

                onReleased: {
                    //console.debug("QML: ItemViewTextInput textFieldMouseArea onReleased with " + rectangle.text)
                }
            }
        }
    }

}
